#include <glpk.h>
#include <iostream>

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/matrix_sparse.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/io.hpp>

#include "fastRationals.hh"

typedef FastRational scalar;
typedef boost::numeric::ublas::mapped_matrix<scalar> myMatrix;
typedef boost::numeric::ublas::mapped_vector<scalar> myVector;
typedef boost::numeric::ublas::matrix_column<myMatrix> myColumn;
typedef boost::numeric::ublas::matrix_row<myMatrix> myRow;
using boost::numeric::ublas::range;
using boost::numeric::ublas::matrix_range;
using boost::numeric::ublas::prod;

unsigned gaussianElim(myMatrix& m, myMatrix& tracker) {
  unsigned i=0;
  for(unsigned j=0; j<m.size2(); j++) {
    myColumn mc(m, j);
    myColumn::const_iterator kIt;
    for(kIt = mc.find(i); kIt != mc.end(); kIt++) {
      if ((*kIt) != 0) break;
    }
    if (kIt == mc.end()) continue;
    unsigned k = kIt.index();
    myRow iRow(m, i);
    myRow iRowTracker(tracker, i);
    if (k != i) {
      iRow.swap(myRow(m, k));
      iRowTracker.swap(myRow(tracker, k));
    }
    const scalar pivot = m(i, j);
    for(kIt = mc.find(i+1); kIt != mc.end(); kIt++) {
      unsigned k = kIt.index();
      scalar alpha = scalar(m(k, j)) / pivot;
      //std::cout << "i=" << i << ", k=" << k << ", alpha=" << alpha << std::endl;
      myRow(m, k) -= alpha * iRow;
      myRow(tracker, k) -= alpha * iRowTracker;
    }
    i++;

    //std::cout << "m = " << m << std::endl;
    //std::cout << "tracker = " << tracker << std::endl;
  }
  return i;
}

myMatrix leftNullSpace(myMatrix m) {
  myMatrix tracker(boost::numeric::ublas::identity_matrix<scalar>(m.size1()));
  unsigned rank = gaussianElim(m, tracker);
  matrix_range<myMatrix> mr(tracker, range(rank, tracker.size1()), range(0, tracker.size2()));
  return myMatrix(mr);
}

int main() {
  myMatrix m(4, 3);
  m(0, 1) = 1;
  m(0, 2) = -1;
  m(1, 0) = 1;
  m(1, 1) = -1;
  m(2, 0) = -1;
  m(2, 2) = 1;
  m(3, 0) = 1;
  m(3, 2) = -1;

  myVector rhs(4);
  rhs(0) = -1;
  rhs(1) = 0;
  rhs(2) = 0;
  rhs(3) = 1;

  myMatrix lns = leftNullSpace(m);
  std::cout << lns << std::endl;
  std::cout << "size = " << (lns.end2()-lns.begin2()) << std::endl;
  std::cout << prod(lns, rhs) << std::endl;

  glp_prob* lp = glp_create_prob();
  glp_set_prob_name(lp, "unsatisfiability witness");
  glp_erase_prob(lp);
}
