#include "smtlib_syntax.hh"
#include "smtlib_lexer.h"
#include <vector>
#include <boost/shared_ptr.hpp>

#ifdef __GNUC__
#define noreturn __attribute__((noreturn))
#else
#define noreturn
#endif

enum sat_status { SAT, UNSAT, UNKNOWN };

struct smtlib_parser {
  enum token last_token;

  enum token get_token() {
    if (last_token == NONE) {
      last_token = smtlib_lex();
    }
    return last_token;
  }

  void next_token() {
    last_token = NONE;
  }

  void parse_formula_list(ManySubFormulas& l);
  void parse_expression_list(ManyExpression& l);

  Formula* parse_formula();
  ManySubFormulas *parse_compound_formula(ManySubFormulas* l);
  Comparison *parse_comparison(Comparison::ComparisonType kind);
  Expression *parse_expression();
  void parse_benchmark();

  boost::shared_ptr <Formula> formula;
  typedef std::vector <boost::shared_ptr <Formula> > assumptions_t;
  assumptions_t assumptions;

  enum sat_status status;

  smtlib_parser() : last_token (NONE), status(UNKNOWN) {
  }

  void parse_error(const std::string& msg) const noreturn;
};
